;-----CODE START-----;
		
		;iNES Header
		.inesprg	1	;# of 16Kb code blocks
		.ineschr	1	;# of 8Kb CHR-ROM blocks (Background 4Kb, Sprites 4Kb)
		.inesmir	1	;Name Table Mirror (1 = Vertical)
		.inesmap	0	;Mapper # 0
		
		;Code
		.bank 0			;Bank where code and variables goes
		.org $0000		;Variables (in CPU RAM)
ScrollPos:	.db 0			;First page of RAM is called Zero Page
A_Pressed:	.db 0			;Faster access and indirect addressing ability
B_Pressed:	.db 0
		.org $0300		;Storing a copy of Sprite RAM here (256 Bytes)
Y_Pos:		.db 0			;will be copied into Sprite RAM via DMA
Tile_Num:	.db 0			;Sprite Attributes go in this order:
Special_Byte:	.db 0			;Sprite1's Y position, Sprite1's Tile Number (in
X_Pos:		.db 0			;CHR-ROM file), Sprite1's Special Byte, and Sprite1's
					;X position. X & Y Position based on top-left corner

		.org $8000		;Where code execution begins
Reset:
		sei			;Disables interrupts (except NMI)
		cld			;Clears Decimal Bit (good practice)
		ldx #$FF		
		txs			;Resets the Stack Pointer to the top (also good practice)

wvb1:					;Wait for 2 Vertical Blanks before doing anything else
		lda $2002
		bpl wvb1
wvb2:
		lda $2002
		bpl wvb2

turnOffScreen:				;Setting $2000 & $2001 to 0 effectively turns off the PPU
		lda #$00
		sta $2000
		sta $2001

		jsr initVariables	;initialize the variables that we're using
		jsr loadPalette		;load the color palette
		jsr loadNameTable1	;load the first screen worth of background (with attributes)
		jsr loadNameTable2	;load the second screen worth of background (with attributes)
		jsr initSound		;initialize sound

wvb3:					;Wait 2 more Vertical Blanks to get our timing right
		lda $2002
		bpl wvb3
wvb4:
		lda $2002
		bpl wvb4

setupScreen:				;Turn the PPU back on and set it up
		lda #$00
		sta $2006
		sta $2006
		lda #%10001000
		sta $2000
		lda #%00011110
		sta $2001

infinite:				;infinite loop to wait for VBlank (NMI) to occur
		jmp infinite

initVariables:
		lda #50
		sta X_Pos		;initialize Sprite1's X Position to 50
		sta Y_Pos		;initialize Sprite1's Y Position to 50
		lda #0
		sta Special_Byte	;don't do anyting special to the sprite
		sta ScrollPos		;initialize background scrolling position to 0
		sta A_Pressed		;used to check if A button was already pressed
		sta B_Pressed		;used to check if B button was already pressed
		lda #1
		sta Tile_Num		;the tile number of Sprite1 in the CHR-ROM
		rts

loadPalette:
		lda #$3F
		sta $2006
		lda #$00
		sta $2006
		ldx #$00
paletteLoop:
		lda colorPalette, x
		sta $2007
		inx
		cpx #32
		bne paletteLoop
		rts

loadNameTable1:
		lda #$20
		sta $2006
		lda #$00
		sta $2006
		ldy #$00
		ldx #$04
		lda #low(nameTable1)
		sta $10
		lda #high(nameTable1)
		sta $11
loadNTLoop1:
		lda [$10], y
		sta $2007
		iny
		bne loadNTLoop1
		inc $11
		dex
		bne loadNTLoop1
		rts

loadNameTable2:
		lda #$24
		sta $2006
		lda #$00
		sta $2006
		ldy #$00
		ldx #$04
		lda #low(nameTable1)
		sta $10
		lda #high(nameTable1)
		sta $11
loadNTLoop2:
		lda [$10], y
		sta $2007
		iny
		bne loadNTLoop2
		inc $11
		dex
		bne loadNTLoop2
		rts

initSound:
		lda #$01
		sta $4015
		lda #$00
		sta $4001
		lda #$40
		sta $4017
		rts

VBlank_Routine:
		lda $2002
		ldx #$00
		stx $2006
		stx $2006
		lda ScrollPos
		sta $2005
		stx $2005

Sprite_DMA:
		lda #$0
		sta $2003
		sta $2003
		lda #$3
		sta $4014

Controller_Strobe:
		lda #$01
		sta $4016
		lda #$00
		sta $4016
A_Check:
		lda $4016
		and #1
		beq A_Not_Pressed
		ldx A_Pressed
		bne B_Check
		sta A_Pressed
		lda #$95
		sta $4000
		lda #$8F
		sta $4001
		lda #$F8
		sta $4002
		lda #$0C
		sta $4003
		jmp B_Check
A_Not_Pressed:
		sta A_Pressed
B_Check:
		lda $4016
		and #1
		beq B_Not_Pressed
		ldx B_Pressed
		bne Select_Check
		sta B_Pressed
		lda #$95
		sta $4000
		lda #$87
		sta $4001
		lda #$80
		sta $4002
		lda #$08
		sta $4003
		jmp Select_Check
B_Not_Pressed:
		sta B_Pressed
Select_Check:
		lda $4016
Start_Check:
		lda $4016
Up_Check:
		lda $4016
		and #1
		beq Down_Check
		lda #7
		cmp Y_Pos
		beq Down_Check
		dec Y_Pos
Down_Check:
		lda $4016
		and #1
		beq Left_Check
		lda #223
		cmp Y_Pos
		beq Left_Check
		inc Y_Pos
Left_Check:
		lda $4016
		and #1
		beq Right_Check
		dec ScrollPos
Right_Check:
		lda $4016
		and #1
		beq RTN_FROM_INT
		inc ScrollPos

RTN_FROM_INT:
		rti


colorPalette:
		.incbin "demo.pal"	;color palette file name here
nameTable1:
		.incbin "demo.map"	;nameTable1 file name here

		
		;Interrupt Handlers
		.bank 1
		.org $FFFA
		.dw VBlank_Routine
		.dw Reset
		.dw RTN_FROM_INT


		;CHR-ROM Data
		.bank 2
		.org $0000
		.incbin	"demo.bkg"	;<background file name here>
		.incbin	"demo.spr"	;<sprite file name here>